@@ -10,15 +10,18 @@
     
     lp0 <- lp
     if (!is.null(offset)) lp <- lp + offset
-    probs <- exp(lp) / (1+exp(lp))
+    explp <- exp(lp)
+    probs <- explp / (1+explp)
     ws <- probs * (1-probs)
 
     # The residuals
     residuals <- response - probs
 
     # The loglikelihood
-    loglik <- sum(log(probs[response == 1])) + sum(log(1-probs[response == 0]))
-    if (!is.na(loglik) && (loglik == - Inf)) loglik <- NA
+    # loglik <- sum(log(probs[response == 1])) + sum(log(1-probs[response == 0]))
+    loglik <- sum(lp[response==1] - log(1+explp[response==1])) + sum(-lp[response==0] - log(1+1/explp[response==0]))
+    
+    if (!is.na(loglik) && (loglik == -Inf)) loglik <- NA
 
     return(list(residuals = residuals, loglik = loglik, W = ws, lp = lp, lp0 = lp0, fitted = probs, nuisance = list()))
   }
